/* Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of simmc-agent
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */

#include <list>
#include "agent/linux/inventory.h"

Json::Value get_inventory() {

    Json::Value inv;

    // os info

    struct utsname sysinfo;
    uname(&sysinfo);

    inv["data"]["os_type"] = "Linux";

    try {
        inv["data"]["os_distro"] = get_distro();
    }
    catch ( std::string err ) {
        inv["data"]["os_distro"] = Json::nullValue;
        inv["error"]["os_distro"] = err;
    }

    inv["data"]["os_kernel"] = sysinfo.release;
    inv["data"]["mirror_timestamp"] = sysinfo.version;

    try {
        inv["data"]["processor"] = get_processor_model();
    }
    catch ( std::string err ) {
        inv["data"]["processor"] = Json::nullValue;
        inv["error"]["processor"] = err;
    }

    try {
        inv["data"]["memory"] = get_memory_size();
    }
    catch ( std::string err ) {
        inv["data"]["memory"] = Json::nullValue;
        inv["error"]["memory"] = err;
    }

    try {
        std::list<disk_t> disks;
        get_disks_info(disks);
        disk_t curr = disks.front();
        inv["data"]["disk1_model"] = curr.model;
        inv["data"]["disk1_size"] = curr.size;
        disks.pop_front();

        if (disks.empty()) {
            inv["data"]["disk2_model"] = Json::nullValue;
            inv["data"]["disk2_size"] = Json::nullValue;
        } else {
            curr = disks.front();
            inv["data"]["disk2_model"] = curr.model;
            inv["data"]["disk2_size"] = curr.size;
            disks.pop_front();
        }

        Json::UInt64 disks_size = disks.size();
        inv["data"]["extra_hds"] = disks_size;

    }
    catch ( std::string err ) {
        inv["data"]["disk1_model"] = Json::nullValue;
        inv["data"]["disk1_size"] = Json::nullValue;
        inv["data"]["disk2_model"] = Json::nullValue;
        inv["data"]["disk2_size"] = Json::nullValue;
        inv["data"]["extra_hds"] = Json::nullValue;

        inv["error"]["disk1_model"] = err;
        inv["error"]["disk1_size"] = err;
        inv["error"]["disk2_model"] = err;
        inv["error"]["disk2_size"] = err;
        inv["error"]["extra_hds"] = err;
    }

    // hw info
    /*
    inv["disk1_used"] =
    inv["disk2_used"] =
    */

    // other
    /*
    inv["agent_version"] =
    inv["contact_date"] =
    */

    if (inv["data"]["os_distro"].asString().find("Linux Comunicações")
                                                != std::string::npos) {
        try {
            inv["data"]["machine_type"] = get_machine_type();
        }
        catch ( std::string err ) {
            inv["data"]["machine_type"] = Json::nullValue;
            inv["error"]["machine_type"] = err;
        }
    } else {
        inv["data"]["machine_type"] = Json::nullValue;
    }

    try {
        inv["data"]["amount_users"] = get_user_count();
    }
    catch ( std::string err ) {
        inv["data"]["amount_users"] = Json::nullValue;
        inv["error"]["amount_users"] = err;
    }

    try {
        inv["data"]["mac_address"] = get_macaddr();
    }
    catch ( std::string err ) {
        inv["data"]["mac_address"] = Json::nullValue;
        inv["error"]["mac_address"] = err;
    }

    return inv;
}
